"""Answer class to represent a Quora answer."""
from quora.content import Content
from quora.question import Question


class Answer:
    def __init__(self, data_dict, user, language="en"):
        """Constructs the `Answer` object."""
        self.id = data_dict.get("id")
        self.aid = data_dict.get("aid")
        self.isPinned = data_dict.get("isPinned")
        self.question = Question(data_dict.get("question"))
        self.language = language
        self.url = "https://www.quora.com" + data_dict.get("permaUrl")
        self.content = Content(data_dict.get("content"))
        self.creationTime = data_dict.get("creationTime")
        self.updatedTime = data_dict.get("updatedTime")
        self.author = data_dict.get("author")
        self.user = user
        self.username = self.user.username

    def __str__(self):
        """Returns the answer text."""
        return self.content.__str__()

    def __eq__(self, other):
        """Returns whether two answers are same or not."""
        return self.aid == other.aid

    def __getstate__(self):
        state = self.__dict__.copy()
        del state["user"]
        return state

    def __setstate__(self, state):
        from .user import User

        self.__dict__.update(state)
        self.user = User(self.username)



"""Data parsers for library."""
import json
import re

from quora.answer import Answer
from quora.topic import Topic
from .exceptions import ProfileNotFoundError


def parse_page(html_data, user):
    """Parse HTML and return JSON."""
    try:
        data = re.search(
            r'window\.ansFrontendGlobals\.data\.inlineQueryResults\.results\[".*\]\.push\((?P<extracted_data>\".*\")\);',
            html_data,
        )  # noqa: E501
        data = data.group('extracted_data')
        data = json.loads(json.loads(data))
        return data["data"]["user"]
    except Exception as e:
        raise ProfileNotFoundError(
            f"No profile\
found with the username {user.username}."
        )


def parse_answers(json_data, user, language):
    """Parse JSON string of answers and return list of `Answer` object."""
    answers = json_data["recentPublicAndPinnedAnswersConnection"]["edges"]
    return [Answer(ans["node"], user, language) for ans in answers]


def parse_topics(json_data, language):
    topics = json_data["expertiseTopicsConnection"]["edges"]
    return [Topic(topic["node"], language) for topic in topics]



from .user import User
from .question import Question
from .answer import Answer
from .content import Content
from .profile import Profile
from .topic import Topic

__all__ = [
    "Answer",
    "Content",
    "Profile",
    "Question",
    "Topic",
    "User",
]



"""Contains Profile class."""
import json


class Profile:
    """Represent the profile of a quora user."""

    def __init__(self, user, data, language="en"):
        """Constructor for Profile."""
        self.user = user
        self.username = user.username
        self.id = data.get("id")
        self.uid = data.get("uid")
        self.language = language
        self.profileImage = data.get("profileImageUrl")
        try:
            self.firstName = data["names"][0]["givenName"]
        except Exception:
            self.firstName = None
        try:
            self.lastName = data["names"][0]["familyName"]
        except Exception:
            self.lastName = None
        try:
            self.profileCrendential = data["profileCredential"]["experience"]
        except Exception:
            self.profileCrendential = None
        try:
            description = json.loads(
                data['descriptionQtextDocument']['legacyJson'])
            description = description['sections']
            bio = ''
            for i in range(len(description)):
                for key in description[i]['spans']:
                    bio += key['text'] + '\n'
            self.profileBio = bio

        except Exception:
            self.profileBio = "Not Available"
        self.contributingSpaceCount = data.get("numCanContributeTribes")
        self.twitterProfileUrl = data.get("twitterProfileUrl")
        self.answerViewsCount = data.get("allTimePublicAnswerViews")
        self.contentViewsCount = data.get("allTimePublicContentViews")
        self.lastMonthContentView = data.get("lastMonthPublicContentViews")
        self.answerCount = data.get("numPublicAnswers")
        self.questionCount = data.get("numProfileQuestions")
        self.shareCount = data.get("quoraSharesCount")
        self.postCount = data.get("numTribePosts", "N/A")
        self.followingCount = data.get("followingCount")
        self.followingSpaceCount = data.get("numFollowedTribes")
        self.followingTopicCount = data.get("numFollowedTopics")
        self.followerCount = data["followerCount"]

    def __str__(self):
        text = ""
        for i, j in vars(self).items():
            text += i + ": " + str(j) + "\n"
        return text

    def __eq__(self, other):
        return vars(self) == vars(other)

    def json(self):
        json = self.__dict__.copy().pop("user")
        return json

    def __getstate__(self):
        state = self.__dict__.copy()
        del state["user"]
        return state

    def __setstate__(self, state):
        from .user import User

        self.__dict__.update(state)
        self.user = User(self.username)



import logging

logger = logging.getLogger(__name__)


def cache(cache_exp=None):
    def decorator(func):
        async def _wrapper(user, *args, **kwargs):
            key = f"pyquora_{func.__name__}_{user.username}_{kwargs.get('language','en')}"
            if user._cache is not None:
                value = user._cache.get(key)
                if value is not None:
                    logger.info(f"Using cache from {key}")
                    return value
            res = await func(user, *args, **kwargs)
            if user._cache is not None:
                time = 0
                if kwargs.get("cache_exp"):
                    time = kwargs.get("cache_exp")
                elif user._cache_exp:
                    time = user._cache_exp
                elif cache_exp:
                    time = cache_exp
                if not isinstance(time, int):
                    raise TypeError(f"cache expiry time should be in int. I got {time} instead.")
                logger.info(f"Storing cache into {key} will expire in {time} seconds")
                user._cache.set(key, res, time=time)
            return res

        return _wrapper

    return decorator



import json


class Content:
    """Represent the text content/
    inside of Question , Answer and other objects."""

    def __init__(self, data_dict):
        """Construts Content."""
        if isinstance(data_dict, str):
            data_dict = json.loads(data_dict)
        self.data_dict = data_dict

    def parse(self, parse_mode="text"):
        """Parses json into text."""
        text = ""
        if parse_mode == "text":
            for section in self.data_dict["sections"]:
                for span in section["spans"]:
                    text += span["text"]
                text += "\n"
        return text

    def __str__(self):
        """Returns the text representation."""
        return self.parse(parse_mode="text")



"""Exception module for library."""


class PyQuoraException(Exception):
    """Base class for exceptions."""


class ProfileNotFoundError(PyQuoraException):
    """Specified Quora profile not found."""



"""Question Class for quora question."""

from quora.content import Content


class Question:
    def __init__(self, data_dict):
        self.id = data_dict.get("id")
        self.qid = data_dict.get("qid")
        self.url = "https://www.quora.com" + data_dict.get("url")
        self.title = Content(data_dict.get("title"))

    def __str__(self):
        return self.title.__str__()

    def __eq__(self, other):
        return self.qid == other.qid



import aiohttp
import asyncio
import logging
from .profile import Profile
from ._parsers import (
    parse_page,
    parse_answers,
    parse_topics,
)
from .cache import cache

subdomains = {
    "en": "English",
    "hi": "हिंदी",
    "es": "Español",
    "fr": "Français",
    "de": "Deutsch",
    "it": "Italiano",
    "ja": "日本語",
    "id": "Indonesia",
    "pt": "Português",
    "nl": "Nederlands",
    "da": "Dansk",
    "fi": "Suomi",
    "no": "Norsk",
    "sv": "Svenska",
    "mr": "मराठी",
    "bn": "বাংলা",
    "ta": "தமிழ்",
    "ar": "العربية",
    "he": "עברית",
    "gu": "ગુજરાતી",
    "kn": "ಕನ್ನಡ",
    "ml": "മലയാളം",
    "te": "తెలుగు",
    "pl": "Polski",
}


class User:
    """Represents a Quora user."""

    def __init__(
        self,
        username,
        session=None,
        logger=logging.getLogger(__name__),
        cache_manager=None,
        cache_exp=None,
    ):
        self.username = username
        self._session = session
        self.logger = logger
        self.htmlLogger = logging.getLogger("pyquora-html")
        self._cache = cache_manager
        self._cache_exp = cache_exp

    def profileUrl(self, language="en"):
        if language == "en":
            apiEndPoint = "https://www.quora.com"
        elif language.lower() in subdomains.keys():
            apiEndPoint = f"https://{language.lower()}.quora.com"
        else:
            raise ValueError(f"{language} language is not found.")
        return apiEndPoint + f"/profile/{self.username}"

    async def _create_session(self) -> None:
        """Creates a aiohttp client session."""
        self._session = aiohttp.ClientSession()

    async def _request(self, url) -> str:
        if self._session is None:
            await self._create_session()
        async with self._session.get(url) as response:
            text = await response.text()
            self.htmlLogger.debug(text)
            return text

    @cache(cache_exp=5)
    async def profile(self, *args, **kwargs):
        """Fetch profile of the user."""
        lang = kwargs.get("language", "en")
        html_data = await self._request(self.profileUrl(language=lang))
        json_data = parse_page(html_data, self)
        return Profile(self, json_data, lang)

    @cache(cache_exp=30)
    async def answers(self, *args, **kwargs):
        """Fetch answers of the User."""
        lang = kwargs.get("language", "en")
        html_data = await self._request(self.profileUrl(language=lang) + "/answers")
        json_data = parse_page(html_data, self)
        answers = parse_answers(json_data, self, lang)
        return answers

    @cache(cache_exp=3600)
    async def knows_about(self, *args, **kwargs):
        """Fetch expertise topics."""
        lang = kwargs.get("language", "en")
        html_data = await self._request(self.profileUrl(language=lang) + "/knows_about")
        json_data = parse_page(html_data, self)
        topics = parse_topics(json_data, lang)
        return topics

    def __eq__(self, other):
        return self.username == other.username


def __del__(self):
    loop = asyncio.get_event_loop()
    task = loop.create_task(self._session.close())
    if not loop.is_running():
        loop.run_untill_complete(task)



class Topic:
    """Class to represent a topic."""

    def __init__(self, json_data, language="en"):
        self.id = json_data.get("id")
        self.name = json_data.get("name")
        self.url = "https://www.quora.com" + json_data.get("url")
        self.language = language
        self.photoUrl = json_data.get("photoUrl")
        self.followerCount = json_data.get("numFollowers")
        self.userAnswersCount = json_data.get("numPublicAnswersOfUser")
        # From August 2021 Quora started using dynamic loading of answers.
        # self.userAnswersUrl = "https://www.quora.com" + json_data.get("userAnswersUrl")

    def __str__(self):
        return self.name

    def __eq__(self, other):
        return self.id == other.id



from setuptools import setup, find_packages


def readme():
    with open(r"README.md") as f:
        README = f.read()
    return README


def requirements():
    with open("requirements.txt", "r") as f:
        requirements = f.read()
    return requirements


setup(
    name="pyquora",
    packages=find_packages(),
    version="0.4.4",
    license="MIT",
    description="""Fetch profiles and data from Quora.""",
    author="Shubhendra Kushwaha",
    author_email="shubhendrakushwaha94@gmail.com",
    url="https://github.com/TheShubhendra/pyquora",
    keywords=[
        "scraper",
        "quora",
    ],
    install_requires=requirements(),
    include_package_data=True,
    long_description=readme(),
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: End Users/Desktop",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries",
        "License :: Free For Educational Use",
        "License :: Free For Home Use",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    python_requires=">=3.7",
)



import asyncio
import inspect
import functools
from .user import User


def _syncify_wrapper(type, method_name):
    method = getattr(type, method_name)

    @functools.wraps(method)
    def _wrapper(*args, **kwargs):
        loop = asyncio.get_event_loop()
        coro = method(*args, **kwargs)
        if loop.is_running():
            return coro
        else:
            return loop.run_until_complete(coro)

    setattr(type, method_name, _wrapper)


def syncify(*types):
    for i in types:
        for m in dir(i):
            if not m.startswith("_") and inspect.iscoroutinefunction(getattr(i, m)):
                _syncify_wrapper(i, m)


syncify(User)

__all__ = [
    "User",
]
